<!-- Copyright 2005-06, Woody/mC and mediaCulture     All rights reserved. -->
<?PHP
/**************************************************** URLMatch.php *****************************************************
************************************************************************************************************************
*** AUTHOR:    Woody/mC, woody@media-Culture.de                                                                      ***
*** DATE:      Sun, 06-Nov-2005                                                                                      ***
*** VERSION:   1.1.0                                                                                                 ***
************************************************************************************************************************
*** PURPOSE:   Provides a regular expression that matches (almost) any kind of Uniform Resource Locator (URL).       ***
***            This RegEx pattern can then be used in find/replace functions which allow the use of regular          ***
***            expressions, e.g. to test user input for validity or to replace the textual representation of an URL  ***
***            by a real hyperlink to that location.                                                                 ***
***                                                                                                                  ***
*** HISTORY:   V1.1.0   Woody/mC   Corrected regular expression in order to accept URLs where the sub-domain part    ***
***                                consists of only one character (were illegal URLs before).                        ***
***                                                                                                                  ***
***            V1.0.0   Woody/mC   Initial version, coded for fun and educational purposes, dedicated to my student  ***
***                                companions of the OvGU, Magdeburg, Germany, and the non-commercial use in the     ***
***                                UniHelp web portal located at "http://www.unihelp.de/".                           ***
***                                                                                                                  ***
*** LICENSE:   Copyright 2005-06, Woody/mC and mediaCulture                                                        ***
***            All rights reserved.                                                                                  ***
***                                                                                                                  ***
***            The contents of this file, as well as all files that come bundled with it (furthermore called "THE    ***
***            SOFTWARE" are open-source freeware and may be used and/or modified according to the terms of the      ***
***            following LICENSE; this also includes any partial work, either in written or computationally          ***
***            processible form.                                                                                     ***
***            Use or modification imply that you accepted all the following conditions (slightly modified, but      ***
***            adapted from the Berkeley BSD license, which can be found at                                          ***
***            "http://www.opensource.org/licenses/bsd-license.php" ):                                               ***
***                                                                                                                  ***
***            Redistribution and use in source and binary forms, with or without modification, are permitted        ***
***            provided that the following conditions are met:                                                       ***
***                                                                                                                  ***
***            (0) Redistributions of source code must retain the above copyright notice, the topmost HTML copyright ***
***                comment, this list of conditions and the following disclaimer.                                                                     ***
***            (1) Redistributions in binary form must reproduce the above copyright notice, this list of conditions ***
***                and the following disclaimer in the documentation and/or other materials provided with the        ***
***                distribution.                                                                                     ***
***            (2) Neither the name and/or logo of mediaCulture and/or mC, the name and/or logo of the program, nor  ***
***                the names of its contributors may be used to endorse or promote products derived from this        ***
***                software without specific prior written permission.                                               ***
***                                                                                                                  ***
***            THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR        ***
***            IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND      ***
***            FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR             ***
***            CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     ***
***            DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     ***
***            DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN ***
***            CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE ***
***            USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                              ***
***                                                                                                                  ***
***            mediaCulture, mC and the mC logo are registered trademarks of the author known by the pseudonym       ***
***            "Woody/mC" and the "mediaCulture" company.                                                            ***
***                                                                                                                  ***
************************************************************************************************************************
***********************************************************************************************************************/





/************************************************* GLOBAL VARIABLES ***************************************************/


/*********************************************************************
*** VARIABLE:   $UM_NoLocalURLs
*** TYPE:       Boolean
*** PURPOSE:    If TRUE, the RegEx pattern described by UM_Pattern
***             matches only those URLs used on the internet that
***             contain a top level domain name (TLD).
***             Otherwise, the pattern matches even local host names
***             (which can be almost any word; use caution!).
*********************************************************************/
$UM_NoLocalURLs = false;


/*********************************************************************
*** VARIABLE:   $UM_NoFileURLs
*** TYPE:       Boolean
*** PURPOSE:    If TRUE, the RegEx pattern described by UM_Pattern
***             matches only those URLs used on the internet that
***             do not reference a local file.
***             Otherwise, the pattern matches also URLs for files
***             locally stored on a disk.
*********************************************************************/
$UM_NoFileURLs = false;





/***************************************************** FUNCTIONS ******************************************************/


/*************************************************************************************
*** FUNCTION:   UM_Pattern()
*** TYPE:       String
*** INPUT:      (nothing)
*** OUTPUT:     A string containing a regular expression (see PURPOSE).
*** PURPOSE:    Returns a regular expression that matches any URL in the form
***             "[protocol://[/]][user[:passwd]@][drive]|[[subdomain.[...]]domain[.tld]]
***             [:port][/dir[...]][/file][#anchor][?query-string]" (thus, "file:///"
***             URLs are not supported).
***             Respectively, also IP addresses can be used instead of DNS names.
***             Obviously, the pattern matches also "[mailto:]user@host"-style
***             email adresses. Rough range checks for IP bytes and port number are
***             performed. The pattern is designed to return the individual fields
***             of an URL in its subpatterns. For more info on this, you should e.g.
***             consult the "preg_match()" function help topic in your PHP manual.
*************************************************************************************/
function UM_Pattern() {

  global $UM_NoLocalURLs, $UM_NoFileURLs;

  $UM_PatternRegEx=
    "/(?:(?i)^".                                      // completely ignore case, begin word
    "(?:([a-z][\da-z]+)\:(?:\/{2".                        // optional protocol with separator "://"...
      (@$UM_NoFileURLs?"":",3")."}))?".                   // ...or ":///" for file URLs

    "(?:([\w]*)".                                     // optional field for username...
        "(?:(?<!^)\:([\w\.\-_]+))?".                      // ...and - if specified - an optional password
        "(?<!^)(?<!\/)@)?".                                            // end of user/password field
                                                          // ANNOT.: the usr/pwd fields mutually exclusive
                                                          // match ("mailto" and ":<user>") or ("<user>" and "")
                                                          // if the given URL is an email address

    "(".                                              // EITHER a domain name...
        "(?:[\da-z][\-\da-z]*\.)*".                       // zero or more subdomain names
        "(?:[\da-z][\-\da-z]+)".                          // domain name
        "(?:\.[a-z]{2,3})".(@$UM_NoLocalURLs?"":"?").     // only make TLD mandatory if local hostnames aren't wanted
    "|".                                              // ...OR a dotted decimal IPv4 address
        "(?:[0-2]?[0-9]{1,2})".                           // four bytes,
        "(?:\.[0-2]?[0-9]{1,2}){3}".                      //  separated by dots
                                                      // (IPv6 support will be added here in later versions)
    "|".                                              // ...OR a drive letter...
        "(?:(?<=(?:\/{3}))(?:[a-z][\:\|])?)".             // ... with a trailing colon or pipe
    ")".                                              // end of domain name / IP / drive letter part

    "(?:\:([0-6]?[0-9]{1,4}))?".                      // optional ":" separator and port

    "(".                                              // complete URI, consisting of:
        "(?:((?:\/[\-\.\_\%\w]+)*)(?:\/)".                // optional directories
        "(?:([^#\\\\\/|><?:*+]+)?)?)?".                   // an optional file name
        "(?:.*\?([^#]*))?".                               // an optional query string
        "(?:\#([\-\_\w\?]*))*".                           // an optional anchor name
    ")".                                              // end of URI part

    "$)/";                                            // end of word

  return $UM_PatternRegEx;
  }


/*************************************************************************************
*** FUNCTION:   UM_ReplaceURLsByHyperlinks()
*** TYPE:       String
*** INPUT:      $text >>> A string where every URL should be replaced by its
***                       appropiate HTML hyperlink notation.
*** OUTPUT:     A string containing the converted text (see PURPOSE).
*** PURPOSE:    Converts every URL in a given (plain) text into its according HTML
***             hyperlink (surrounded by an opening as well as a closing <A> tag).
***             This function should be modified if BBcode behaviour needs to be
***             simulated. Simply replace the "<A HREF..." by somthing like "[url]"
***             and the "</A>" tag by "[/url]" (or whatever you need), "\\0" stands
***             for the complete URL if successfully matched.
***             To accomplish this task, "UM_ReplaceURLsByHyperlinks()" first splits
***             the incoming text into an array of words (using whitespace characters
***             as a separator), checks every word if it represents an URL (using the
***             pattern defined by "UM_Pattern()"), and, if it is, it prepends and
***             appends the required tags. Finally, the array is rejoint using a
***             single blank (ASCII #32) and returned as a string. Please note that
***             the output string does NOT contain the original line break characters
***             and that URLs might not be catched if they have leading characters.
*************************************************************************************/
function UM_ReplaceURLsByLinks($text) {

  global $UM_NoLocalURLs;

  // Split the text into words
  $UM_Words = preg_split("/[\s\"]+/", $text, -1, PREG_SPLIT_NO_EMPTY);

  // Disable local URLs , otherwise nearly any word would be matched
  $UM_NoLocalURLs = true;

  // Optimization: Cache pattern, so it doesn't have to be recalculated every time
  $p = UM_Pattern();


  // Replace URLs word by word
  while (list($i,$s)=each($UM_Words)) {

    // Try to extract a valid URL from the current word by removing trailing chars.
    //  These need to be reappended after a successful replacement if a valid URL
    //  has been found during the stripping process.
    $CharsToAppend = "";
    while ( !preg_match($p, $s) && ($s != "") ) {
      $CharsToAppend = substr($s, -1).$CharsToAppend;
      $s = substr($s, 0, -1);
      }

    // Replace only if we don't deal with en empty word (which obviously is no URL).
    if ($s != "") {

      // If the protocol is omitted, the URL might either specify an unprepended e-Mail
      //  address or a plain HTTP URL (as any browser processes).
      if (strpos($s, "://") === false) {


        // If the protocol is omitted but the word contains an "@", it's most likely an
        //  unprepended e-Mail address. In that case, the HREF needs the "mailto:" prefix
        //  in order to respond properly. Otherwise, if neither the protocol is specified
        //  nor an "@" was found, the browser should handle it as a plain HTTP URL.
        if ((strpos($s, "mailto:") === false) && (strpos($s, "@") > 0))

          // URL seems to be an unprepended eMail address. To let your browser open its
          //  "Compose Mail..." window, prepend the correct protocol ("mailto:").
          $s = preg_replace($p, "<A HREF=\"mailto:\\0\">\\0</A>", $s);
        else
          // URL is plain HTTP, but would be relative to the current directory.
          //  To turn it into an absolute one, prepend the protocol ("http://").
          $s = preg_replace($p, "<A HREF=\"http://\\0\">\\0</A>", $s);

        }  // Finished prepending omitted protocol information

      else

        // The protocol information was specified correctly, leave the URL unchanged.
        $s = preg_replace($p, "<A HREF=\"\\0\">\\0</A>", $s);

      }  // End of conditional replacements

    // If characters have been stripped off the end, reappend these to the word
    if ($CharsToAppend != "")
      $s .= $CharsToAppend;

    // Save changes to the word array
    $UM_Words[$i] = $s;

    }  // Replacement finished for a single word


  // We're all done - glue the words together again
  return join(" ", $UM_Words);
  }





/**************************************************** MAIN PROGRAM ****************************************************/


// Some sample code to demonstrate the use of the defined pattern and the replacement function.
// Put this file on a PHP-powered web server and call <url-of-this-file>?<some-text> to see
// whether <some-text> is recognized as an URL.
// To use this file as an includable library, you should remove the sample code.

$text="Here you can see some of the functionality <B> URLMatch.php </B> provides. <BR><BR>".
       "Although this is free software, it would be nice to get a little reward which does not ".
       "necessarily need to be a material one, an entry in my private guestbook which can be ".
       "found at http://home.media-Culture.de would already do it... If you have questions or ".
       "comments, you can write an e-Mail to support@media-Culture.de?subject=URLMatch.php .";

echo "<H1>URLMatch.php</H1>".UM_ReplaceURLsByLinks($text)."<BR><BR><HR>";

echo "\"".$_SERVER["QUERY_STRING"]."\" is ".
      (preg_match(UM_Pattern(),$_SERVER["QUERY_STRING"],$UM_Result)?"an":"no")." URL!";

echo "<HR><BR><BR>Sub-patterns as returned by <B>preg_match()</B>:<BR><PRE>";
while (list($i,$s)=each($UM_Result))
  echo "\$UM_Result[".$i."] = \t\"".$s."\"<BR>";
echo (count(@$UM_Result)?"":"(none)")."</PRE>";

?>